package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.entity.UserMappingInfoRole;
import com.abi.qms.platform.dao.entity.UserRole;
import com.abi.qms.platform.dao.enums.BaseFinal;
import com.abi.qms.platform.dao.enums.ErrorCodeEnum;
import com.abi.qms.platform.dao.enums.RedisKey;
import com.abi.qms.platform.dao.mapper.UserInfoMapper;
import com.abi.qms.platform.dao.mapper.UserMappingInfoRoleMapper;
import com.abi.qms.platform.dao.mapper.UserRoleMapper;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.GetUserInfoRes;
import com.abi.qms.platform.dto.res.ListUserRes;
import com.abi.qms.platform.dto.res.LoginRes;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.RedisUtils;
import com.abi.qms.platform.infrastructure.util.UUIDutils;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.UserManagerService;
import com.abi.task.common.api.base.BaseResponse;
import com.abi.task.common.utils.AESEncodeTwoUtil;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 用户管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@Service
public class UserManagerServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements UserManagerService {

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private UserMappingInfoRoleMapper userMappingInfoRoleMapper;

    /**
     * 创建用户
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse createUser(CreateUserReq createUserReq) {
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(createUserReq, userInfo);
        QueryWrapper<UserRole> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_code", userInfo.getRoleCode());
        roleQueryWrapper.eq("is_delete", 0);
        UserRole userRole = userRoleMapper.selectOne(roleQueryWrapper);
        if (Objects.isNull(userRole)) {
            return BaseResponse.createFail(null, ErrorCodeEnum.USER_ROLE_NOT_EXISTED.getCode(), ErrorCodeEnum.USER_ROLE_NOT_EXISTED.getMessage());
        }
        if (lambdaQuery().eq(UserInfo::getEmployeeNo, userInfo.getEmployeeNo().trim()).count() > 0) {
            log.info("工号" + userInfo.getEmployeeNo() + "已存在，请勿重复创建。");
            return BaseResponse.create(null, ErrorCodeEnum.EMPLOYEE_NO_DUPLICATE.getCode(), "工号" + userInfo.getEmployeeNo() + "已存在，请勿重复创建。");
        }
        if (lambdaQuery().eq(UserInfo::getUserName, userInfo.getUserName().trim()).count() > 0) {
            log.info("用户" + userInfo.getUserName() + "已存在，请勿重复创建。");
            return BaseResponse.create(null, ErrorCodeEnum.USERNAME_DUPLICATE.getCode(), "用户" + userInfo.getUserName() + "已存在，请勿重复创建。");
        }
        UserInfo userInfo1 = UserUtil.getUser();
        userInfo.setCreateBy(userInfo1.getId());
        save(userInfo);
        //添加用户和角色的映射关系
        QueryWrapper<UserMappingInfoRole> qwUR = new QueryWrapper<>();
        qwUR.eq("user_id", userInfo.getId());
        qwUR.eq("is_delete", 0);
        UserMappingInfoRole userMappingInfoRole = userMappingInfoRoleMapper.selectOne(qwUR);
        if(Objects.nonNull(userMappingInfoRole)){
            userMappingInfoRole.setRoleCode(userRole.getRoleCode());
            userMappingInfoRole.setRoleName(userRole.getRoleName());
            userMappingInfoRoleMapper.updateById(userMappingInfoRole);
        }else{
            userMappingInfoRole = new UserMappingInfoRole();
            userMappingInfoRole.setUserId(userInfo.getId());
            userMappingInfoRole.setRoleCode(userRole.getRoleCode());
            userMappingInfoRole.setRoleName(userRole.getRoleName());
            userMappingInfoRoleMapper.insert(userMappingInfoRole);
        }
        return BaseResponse.create();
    }

    /**
     * 修改用户
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse updateUser(UpdateUserReq updateUserReq) {
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(updateUserReq, userInfo);
        QueryWrapper<UserRole> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_code", userInfo.getRoleCode());
        roleQueryWrapper.eq("is_delete", 0);
        UserRole userRole = userRoleMapper.selectOne(roleQueryWrapper);
        if (Objects.isNull(userRole)) {
            return BaseResponse.createFail(null, ErrorCodeEnum.USER_ROLE_NOT_EXISTED.getCode(), ErrorCodeEnum.USER_ROLE_NOT_EXISTED.getMessage());
        }
        userInfoMapper.updateById(userInfo);
        //添加用户和角色的映射关系
        QueryWrapper<UserMappingInfoRole> qwUR = new QueryWrapper<>();
        qwUR.eq("user_id", userInfo.getId());
        qwUR.eq("is_delete", 0);
        UserMappingInfoRole userMappingInfoRole = userMappingInfoRoleMapper.selectOne(qwUR);
        if(Objects.nonNull(userMappingInfoRole)){
            userMappingInfoRole.setRoleCode(userRole.getRoleCode());
            userMappingInfoRole.setRoleName(userRole.getRoleName());
            userMappingInfoRoleMapper.updateById(userMappingInfoRole);
        }else{
            userMappingInfoRole = new UserMappingInfoRole();
            userMappingInfoRole.setUserId(userInfo.getId());
            userMappingInfoRole.setRoleCode(userRole.getRoleCode());
            userMappingInfoRole.setRoleName(userRole.getRoleName());
            userMappingInfoRoleMapper.insert(userMappingInfoRole);
        }
        return BaseResponse.create();
    }

    /**
     * 查询单个用户信息
     **/
    @Override
    public GetUserInfoRes getUserInfo(Long userInfoId) {
        UserInfo userInfo = userInfoMapper.selectById(userInfoId);
        GetUserInfoRes getUserInfoRes = new GetUserInfoRes();
        BeanUtils.copyProperties(userInfo, getUserInfoRes);
        return getUserInfoRes;
    }

    /**
     * 查询用户列表
     **/
    @Override
    public ListUserRes listUser(ListUserReq listUserReq) {
        IPage iPage = userInfoMapper.selectUserInfoPage(PageUtil.createPage(listUserReq), listUserReq);
        List<UserInfo> records = iPage.getRecords();
        //封装出参
        ListUserRes res = new ListUserRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListUserRes.UserInfoBean> eventBeanList = PojoConverterUtils.copyList(records, ListUserRes.UserInfoBean.class);
        res.setUserInfoBeanList(eventBeanList);
        return res;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void enableUser(Long userInfoId) {
        UserInfo userInfo = userInfoMapper.selectById(userInfoId);
        if (ObjectUtil.isNotEmpty(userInfo)) {
            userInfo.setIsEnable(1 - userInfo.getIsEnable());
            userInfoMapper.updateById(userInfo);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchEnableUser(BatchEnableUserReq req) throws Exception {
        userInfoMapper.batchEnableUser(req);
    }


}
